@@ -1,21 +1,17 @@
 """
 Generic Non-Maximum Suppression API with efficient backend implementations
 """
+import sys
 import numpy as np
 import ubelt as ub
 import warnings
 import kwarray
 
-try:
-    import torch
-except Exception:
-    torch = None
-
 
 def daq_spatial_nms(ltrb, scores, diameter, thresh, max_depth=6,
                     stop_size=2048, recsize=2048, impl='auto', device_id=None):
     """
-    Divide and conquor speedup non-max-supression algorithm for when bboxes
+    Divide and conquer speedup non-max-supression algorithm for when bboxes
     have a known max size
 
     Args:
@@ -32,21 +28,27 @@ def daq_spatial_nms(ltrb, scores, diameter, thresh, max_depth=6,
             greater than this threshold. 0 is the most strict, resulting in the
             fewest boxes, and 1 is the most permissive resulting in the most.
 
-        max_depth (int): maximum number of times we can divide and conquor
+        max_depth (int): maximum number of times we can divide and conquer
 
         stop_size (int): number of boxes that triggers full NMS computation
 
         recsize (int): number of boxes that triggers full NMS recombination
 
-        impl (str): algorithm to use
+        impl (str): algorithm to use. Defaults to "auto". Can be "python",
+            "cython_cpu", "gpu", "torch", or "torchvision".
+
+    Note:
+
+        # TODO: Look Into
 
-    LookInfo:
         # Didn't read yet but it seems similar
+
         http://www.cyberneum.de/fileadmin/user_upload/files/publications/CVPR2010-Lampert_[0].pdf
 
         https://www.researchgate.net/publication/220929789_Efficient_Non-Maximum_Suppression
 
         # This seems very similar
+
         https://projet.liris.cnrs.fr/m2disco/pub/Congres/2006-ICPR/DATA/C03_0406.PDF
 
     Example:
@@ -195,13 +197,20 @@ class _NMS_Impls():
     def _lazy_init(self):
         _funcs = {}
         from kwimage import _internal
+        try:
+            import torch
+        except ImportError:
+            torch = None
 
         # These are pure python and should always be available
         from kwimage.algo._nms_backend import py_nms
         from kwimage.algo._nms_backend import torch_nms
         _funcs['numpy'] = py_nms.py_nms
 
-        from distutils.version import LooseVersion
+        try:
+            from packaging.version import parse as LooseVersion
+        except ImportError:
+            from distutils.version import LooseVersion
         recent_numpy = LooseVersion(np.__version__) >= LooseVersion('1.20.0')
 
         if torch is not None and recent_numpy:
@@ -212,7 +221,7 @@ class _NMS_Impls():
                 # why we have an option to disable even trying it
                 try:
                     # TODO: torchvision impl might be the best, need to test
-                    from torchvision import _C as C  # NOQA
+                    # from torchvision import _C as C  # NOQA
                     import torchvision
                     _funcs['torchvision'] = torchvision.ops.nms
                 except (ImportError, UnicodeDecodeError) as ex:
@@ -243,8 +252,8 @@ class _NMS_Impls():
                         # See the benchmarks for more info.
                         # ~/code/kwimage/dev/bench_nms.py
             except Exception as ex:
-                warnings.warn
-                ('optional gpu_nms is not available: {}'.format(str(ex)))
+                warnings.warn(
+                    'optional gpu_nms is not available: {}'.format(str(ex)))
         self._funcs = _funcs
         self._valid = frozenset(_impls._funcs.keys())
 
@@ -260,6 +269,7 @@ def available_nms_impls():
         xdoctest -m kwimage.algo.algo_nms available_nms_impls
 
     Example:
+        >>> from kwimage.algo.algo_nms import *  # NOQA
         >>> impls = available_nms_impls()
         >>> assert 'numpy' in impls
         >>> print('impls = {!r}'.format(impls))
@@ -272,7 +282,7 @@ def available_nms_impls():
 # @ub.memoize
 def _heuristic_auto_nms_impl(code, num, valid=None):
     """
-    Defined with help from ``~/code/kwimage/dev/bench_nms.py``
+    Defined with help from ``~/code/kwimage/dev/bench/bench_nms.py``
 
     Args:
         code (str): text that indicates which type of data you have
@@ -287,7 +297,7 @@ def _heuristic_auto_nms_impl(code, num, valid=None):
     Ignore:
         _impls._funcs
         valid_pref = ub.oset(preference) & set(_impls._funcs.keys())
-        python ~/code/kwimage/dev/bench_nms.py --show --small-boxes --thresh=0.6
+        python ~/code/kwimage/dev/bench/bench_nms.py --show --small-boxes --thresh=0.6
     """
     if code not in {'tensor0', 'tensor', 'ndarray'}:
         raise KeyError(code)
@@ -362,6 +372,8 @@ def non_max_supression(ltrb, scores, thresh, bias=0.0, classes=None,
     """
     Non-Maximum Suppression - remove redundant bounding boxes
 
+    Based on information from [CythonNMS]_ and [NMSPython]_.
+
     Args:
         ltrb (ndarray[Any, Float32]):
             Float32 array of shape Nx4 representing boxes in ltrb format
@@ -381,7 +393,9 @@ def non_max_supression(ltrb, scores, thresh, bias=0.0, classes=None,
             If specified NMS is done on a perclass basis.
 
         impl (str): implementation can be "auto", "python", "cython_cpu",
-            "gpu", "torch", or "torchvision".
+            "gpu", "torch", or "torchvision". Not all backends may be
+            available, see :func:`kwimage.algo.algo_nms.available_nms_impls`
+            for what is supported on your system.
 
         device_id (int): used if impl is gpu, device id to work on. If not
             specified `torch.cuda.current_device()` is used.
@@ -392,10 +406,18 @@ def non_max_supression(ltrb, scores, thresh, bias=0.0, classes=None,
         impl='cython_gpu', and you feed it too many bounding boxes. Ideally this will
         be fixed in the future.
 
+        TODO: SoftNMS [SoftNMS]_.
+
     References:
-        https://github.com/facebookresearch/Detectron/blob/master/detectron/utils/cython_nms.pyx
-        https://www.pyimagesearch.com/2015/02/16/faster-non-maximum-suppression-python/
-        https://github.com/bharatsingh430/soft-nms/blob/master/lib/nms/cpu_nms.pyx <- TODO
+
+        .. [CythonNMS] https://github.com/facebookresearch/Detectron/blob/master/detectron/utils/cython_nms.pyx
+
+        .. [NMSPython] https://www.pyimagesearch.com/2015/02/16/faster-non-maximum-suppression-python/
+
+        .. [SoftNMS] https://github.com/bharatsingh430/soft-nms/blob/master/lib/nms/cpu_nms.pyx
+
+    SeeAlso:
+        :func:`kwimage.structs.detections.Detections.non_max_supression`.
 
     CommandLine:
         xdoctest -m ~/code/kwimage/kwimage/algo/algo_nms.py non_max_supression
@@ -467,7 +489,7 @@ def non_max_supression(ltrb, scores, thresh, bias=0.0, classes=None,
         >>>     keep = non_max_supression(ltrb, scores, thresh, impl=impl)
         >>>     solutions[impl] = sorted(keep)
         >>> assert 'numpy' in solutions
-        >>> print('solutions = {}'.format(ub.repr2(solutions, nl=1)))
+        >>> print('solutions = {}'.format(ub.urepr(solutions, nl=1)))
         >>> assert ub.allsame(solutions.values())
 
     CommandLine:
@@ -490,24 +512,20 @@ def non_max_supression(ltrb, scores, thresh, bias=0.0, classes=None,
         >>>     keep = non_max_supression(ltrb, scores, thresh, impl=impl)
         >>>     solutions[impl] = sorted(keep)
         >>> assert 'numpy' in solutions
-        >>> print('solutions = {}'.format(ub.repr2(solutions, nl=1)))
+        >>> print('solutions = {}'.format(ub.urepr(solutions, nl=1)))
         >>> assert ub.allsame(solutions.values())
     """
-
     if impl == 'cpu':
-        import warnings
         warnings.warn(
             'impl="cpu" is deprecated use impl="cython_cpu" instead',
             DeprecationWarning)
-        impl = 'cython_impl'
+        impl = 'cython_cpu'
     elif impl == 'gpu':
-        import warnings
         warnings.warn(
             'impl="gpu" is deprecated use impl="cython_gpu" instead',
             DeprecationWarning)
         impl = 'cython_gpu'
     elif impl == 'py':
-        import warnings
         warnings.warn(
             'impl="py" is deprecated use impl="numpy" instead',
             DeprecationWarning)
@@ -516,6 +534,8 @@ def non_max_supression(ltrb, scores, thresh, bias=0.0, classes=None,
     if not _impls._funcs:
         _impls._lazy_init()
 
+    torch = sys.modules.get('torch', None)
+
     if ltrb.shape[0] == 0:
         return []
 
