@@ -3,7 +3,6 @@ Not sure how to best classify these functions
 """
 import ubelt as ub
 import numpy as np
-import math
 
 
 def num_channels(img):
@@ -242,12 +241,19 @@ def atleast_3channels(arr, copy=True):
     Returns:
         ndarray: with shape (N, M, C), where C in {3, 4}
 
+    SeeAlso:
+        * :func:`exactly_1channel`
+        * :func:`atleast_3channels`
+        * :func:`kwarray.atleast_nd`
+
     Doctest:
         >>> assert atleast_3channels(np.zeros((10, 10))).shape[-1] == 3
         >>> assert atleast_3channels(np.zeros((10, 10, 1))).shape[-1] == 3
         >>> assert atleast_3channels(np.zeros((10, 10, 3))).shape[-1] == 3
         >>> assert atleast_3channels(np.zeros((10, 10, 4))).shape[-1] == 4
     """
+    # TODO: add argument that speifies which dimension is the channel
+    # dimension.
     ndims = len(arr.shape)
     if ndims == 2:
         res = np.tile(arr[:, :, None], 3)
@@ -265,12 +271,62 @@ def atleast_3channels(arr, copy=True):
     return res
 
 
+def exactly_1channel(image, ndim=2):
+    """
+    Returns a 1-channel image as either a 2D or 3D array.
+
+    Args:
+        image (ndarray): an image with shape (H, W, 1) or (H, W).
+
+        ndim (int): number of dimensions in the output array.
+            Can be either 2 or 3.
+
+    Returns:
+        ndarray:
+            if ndim is 2, returns a (H, W) image.
+            if ndim is 3, returns a (H, W, 1) image.
+
+    Raises:
+        ValueError: if assumptions are not met.
+
+    SeeAlso:
+        * :func:`exactly_1channel`
+        * :func:`atleast_3channels`
+        * :func:`kwarray.atleast_nd`
+
+    Example:
+        >>> import kwimage
+        >>> assert kwimage.exactly_1channel(np.empty((3, 3)), ndim=2).shape == (3, 3)
+        >>> assert kwimage.exactly_1channel(np.empty((3, 3)), ndim=3).shape == (3, 3, 1)
+        >>> assert kwimage.exactly_1channel(np.empty((3, 3, 1)), ndim=2).shape == (3, 3)
+        >>> assert kwimage.exactly_1channel(np.empty((3, 3, 1)), ndim=3).shape == (3, 3, 1)
+        >>> import pytest
+        >>> with pytest.raises(ValueError):
+        >>>     kwimage.exactly_1channel(np.empty((3, 3, 2)), ndim=2)
+        >>> with pytest.raises(ValueError):
+        >>>     kwimage.exactly_1channel(np.empty((3)), ndim=3)
+    """
+    if len(image.shape) == 3:
+        if image.shape[2] != 1:
+            raise ValueError('Expected a 3D 1 channel image')
+        if ndim == 2:
+            image = image[:, :, 0]
+        else:
+            assert ndim == 3
+    else:
+        if len(image.shape) != 2:
+            raise ValueError('Expected a 2D 1 channel image')
+        if ndim == 3:
+            image = image[:, :, None]
+    return image
+
+
 def padded_slice(data, in_slice, pad=None, padkw=None, return_info=False):
     """
     Allows slices with out-of-bound coordinates.  Any out of bounds coordinate
     will be sampled via padding.
 
-    DEPRECATED FOR THE VERSION IN KWARRAY (slices are more array-ish than
+    REMOVED FOR THE VERSION IN KWARRAY (slices are more array-ish than
     image-ish)
 
     Note:
@@ -296,48 +352,11 @@ def padded_slice(data, in_slice, pad=None, padkw=None, return_info=False):
 
         return_info (bool): if True, return extra information
             about the transform. Defaults to False.
-
-    SeeAlso:
-        _padded_slice_embed - finds the embedded slice and padding
-        _padded_slice_apply - applies padding to sliced data
-
-    Returns:
-
-        Sliceable:
-            data_sliced: subregion of the input data (possibly with padding,
-                depending on if the original slice went out of bounds)
-
-
-        Tuple[Sliceable, Dict] :
-            data_sliced : as above
-
-            transform : information on how to return to the original coordinates
-
-                Currently a dict containing:
-                    st_dims: a list indicating the low and high space-time
-                        coordinate values of the returned data slice.
-
-                The structure of this dictionary mach change in the future
-
-    Example:
-        >>> data = np.arange(5)
-        >>> in_slice = [slice(-2, 7)]
-
-        >>> data_sliced = padded_slice(data, in_slice)
-        >>> print(ub.repr2(data_sliced, with_dtype=False))
-        np.array([0, 0, 0, 1, 2, 3, 4, 0, 0])
-
-        >>> data_sliced = padded_slice(data, in_slice, pad=(3, 3))
-        >>> print(ub.repr2(data_sliced, with_dtype=False))
-        np.array([0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 0, 0, 0, 0, 0])
-
-        >>> data_sliced = padded_slice(data, slice(3, 4), pad=[(1, 0)])
-        >>> print(ub.repr2(data_sliced, with_dtype=False))
-        np.array([2, 3])
     """
-    import warnings
-    warnings.warn('Deprecated use kwarray.padded_slice instead',
-                  DeprecationWarning)
+    ub.schedule_deprecation(
+        'kwimage', 'padded_slice', 'function',
+        migration='use kwarray.padded_slice instead',
+        deprecate='now', error='0.11.0', remove='0.12.0')
 
     if isinstance(in_slice, slice):
         in_slice = [in_slice]
@@ -503,6 +522,10 @@ def normalize(arr, mode='linear', alpha=None, beta=None, out=None):
         DEPRECATED: this function has been MOVED to ``kwarray.normalize``
     """
     import kwarray
+    ub.schedule_deprecation(
+        'kwimage', 'normalize', 'function',
+        migration='use kwarray.normalize instead',
+        deprecate='0.10.0', error='0.11.0', remove='0.12.0')
     return kwarray.normalize(arr, mode=mode, alpha=alpha, beta=beta, out=out)
 
 
@@ -510,109 +533,27 @@ def find_robust_normalizers(data, params='auto'):
     """
     Finds robust normalization statistics for a single observation
 
-    Args:
-        data (ndarray): a 1D numpy array where invalid data has already been removed
-
-        params (str | dict): normalization params
-
-    Returns:
-        Dict[str, str | float]: normalization parameters
-
-    TODO:
-        - [ ] No Magic Numbers! Use first principles to deterimine defaults.
-        - [ ] Probably a lot of literature on the subject.
-        - [ ] Is this a kwarray function in general?
-
-    Example:
-        >>> from kwimage.im_core import *  # NOQA
-        >>> data = np.random.rand(100)
-        >>> norm_params1 = find_robust_normalizers(data, params='auto')
-        >>> norm_params2 = find_robust_normalizers(data, params={'low': 0, 'high': 1.0})
-        >>> norm_params3 = find_robust_normalizers(np.empty(0), params='auto')
-        >>> print('norm_params1 = {}'.format(ub.repr2(norm_params1, nl=1)))
-        >>> print('norm_params2 = {}'.format(ub.repr2(norm_params2, nl=1)))
-        >>> print('norm_params3 = {}'.format(ub.repr2(norm_params3, nl=1)))
+    REMOVED IN FAVOR of kwarray.find_robust_normalizers
     """
-    if data.size == 0:
-        normalizer = {
-            'type': None,
-            'min_val': np.nan,
-            'max_val': np.nan,
-        }
-    else:
-        # should center the desired distribution to visualize on zero
-        # beta = np.median(imdata)
-        default_params = {
+    import kwarray
+    ub.schedule_deprecation(
+        'kwimage', 'find_robust_normalizers', 'function',
+        migration=ub.paragraph(
+            '''
+            use kwarray.find_robust_normalizers instead.  Note: the default
+            here is params=sigmoid, but in kwarray params default to linear.
+            Modify the code accordintly.
+            '''),
+        deprecate='0.10.0', error='0.11.0', remove='0.12.0')
+    if isinstance(params, str) and params == 'auto':
+        # Override kwarray defaults for backwards compatability
+        params = {
             'low': 0.01,
             'mid': 0.5,
             'high': 0.9,
             'mode': 'sigmoid',
         }
-        if isinstance(params, str):
-            if params == 'auto':
-                params = {}
-            else:
-                raise KeyError(params)
-
-        params = ub.dict_union(default_params, params)
-        quant_low = params['low']
-        quant_mid = params['mid']
-        quant_high = params['high']
-        qvals = [0, quant_low, quant_mid, quant_high, 1]
-        quantile_vals = np.quantile(data, qvals)
-
-        (quant_low_abs, quant_low_val, quant_mid_val, quant_high_val,
-         quant_high_abs) = quantile_vals
-
-        # TODO: we could implement a hueristic where we do a numerical inspection
-        # of the intensity distribution. We could apply a normalization that is
-        # known to work for data with that sort of histogram distribution.
-        # This might involve fitting several parametarized distributions to the
-        # data and choosing the one with the best fit. (check how many modes there
-        # are).
-
-        # inner_range = quant_high_val - quant_low_val
-        # upper_inner_range = quant_high_val - quant_mid_val
-        # upper_lower_range = quant_mid_val - quant_low_val
-        # http://mathcenter.oxford.emory.edu/site/math117/shapeCenterAndSpread/
-
-        # Compute amount of weight in each quantile
-        quant_center_amount = (quant_high_val - quant_low_val)
-        quant_low_amount = (quant_mid_val - quant_low_val)
-        quant_high_amount = (quant_high_val - quant_mid_val)
-
-        if math.isclose(quant_center_amount, 0):
-            high_weight = 0.5
-            low_weight = 0.5
-        else:
-            high_weight = quant_high_amount / quant_center_amount
-            low_weight = quant_low_amount / quant_center_amount
-
-        quant_high_residual = (1.0 - quant_high)
-        quant_low_residual = (quant_low - 0.0)
-        # todo: verify, having slight head fog, not 100% sure
-        low_pad_val = quant_low_residual * (low_weight * quant_center_amount)
-        high_pad_val = quant_high_residual * (high_weight * quant_center_amount)
-
-        min_val = max(quant_low_abs, quant_low_val - low_pad_val)
-        max_val = max(quant_high_abs, quant_high_val - high_pad_val)
-
-        beta = quant_mid_val
-        # division factor
-        # from scipy.special import logit
-        # alpha = max(abs(old_min - beta), abs(old_max - beta)) / logit(0.998)
-        # This chooses alpha such the original min/max value will be pushed
-        # towards -1 / +1.
-        alpha = max(abs(min_val - beta), abs(max_val - beta)) / 6.212606
-
-        normalizer = {
-            'type': 'normalize',
-            'mode': params['mode'],
-            'min_val': min_val,
-            'max_val': max_val,
-            'beta': beta,
-            'alpha': alpha,
-        }
+    normalizer = kwarray.find_robust_normalizers(data, params=params)
     return normalizer
 
 
@@ -626,7 +567,7 @@ def normalize_intensity(imdata, return_info=False, nodata=None, axis=None,
     reasonable for visualization.
 
     TODO:
-        - [ ] Move to kwarray and renamed to robust_normalize?
+        - [x] Move to kwarray and renamed to robust_normalize?
         - [ ] Support for M-estimators?
 
     Args:
@@ -638,6 +579,7 @@ def normalize_intensity(imdata, return_info=False, nodata=None, axis=None,
 
         params (str | dict):
             can contain keys, low, high, or center
+            e.g. {'low': 0.1, 'center': 0.8, 'high': 0.9}
 
         axis (None | int):
             The axis to normalize over, if unspecified, normalize jointly
@@ -658,6 +600,7 @@ def normalize_intensity(imdata, return_info=False, nodata=None, axis=None,
         ndarray: a floating point array with values between 0 and 1.
 
     Example:
+        >>> # xdoctest: +REQUIRES(module:cv2)
         >>> from kwimage.im_core import *  # NOQA
         >>> import ubelt as ub
         >>> import kwimage
@@ -677,7 +620,7 @@ def normalize_intensity(imdata, return_info=False, nodata=None, axis=None,
         >>> forground = (kwimage.ensure_float01(forground) * max_val).astype(dtype)
         >>> imdata = background + forground
         >>> normed, info = normalize_intensity(imdata, return_info=True)
-        >>> print('info = {}'.format(ub.repr2(info, nl=1)))
+        >>> print('info = {}'.format(ub.urepr(info, nl=1)))
         >>> # xdoctest: +REQUIRES(--show)
         >>> import kwplot
         >>> kwplot.autompl()
@@ -701,10 +644,10 @@ def normalize_intensity(imdata, return_info=False, nodata=None, axis=None,
         >>> rows = []
         >>> rows.append({'key': 'orig', 'result': imdata})
         >>> for params in quantile_grid:
-        >>>     key = ub.repr2(params, compact=1)
+        >>>     key = ub.urepr(params, compact=1)
         >>>     result, info = normalize_intensity(imdata, return_info=True, params=params)
         >>>     print('key = {}'.format(key))
-        >>>     print('info = {}'.format(ub.repr2(info, nl=1)))
+        >>>     print('info = {}'.format(ub.urepr(info, nl=1)))
         >>>     rows.append({'key': key, 'info': info, 'result': result})
         >>> # xdoctest: +REQUIRES(--show)
         >>> import kwplot
@@ -715,67 +658,196 @@ def normalize_intensity(imdata, return_info=False, nodata=None, axis=None,
         >>>     ax.set_title(row['key'])
     """
     import kwarray
+    if isinstance(params, str) and params == 'auto':
+        # Override kwarray defaults for backwards compatability
+        params = {
+            'low': 0.01,
+            'mid': 0.5,
+            'high': 0.9,
+            'mode': 'sigmoid',
+        }
+    return kwarray.robust_normalize(imdata, return_info=return_info,
+                                    nodata=nodata, axis=axis, dtype=dtype,
+                                    params=params, mask=mask)
 
-    if axis is not None:
-        # Hack, normalize each channel individually. This could
-        # be implementd more effciently.
-        assert not return_info
-        reorg = imdata.swapaxes(0, axis)
-        if mask is None:
-            parts = []
-            for item in reorg:
-                part = normalize_intensity(item, nodata=nodata, axis=None)
-                parts.append(part[None, :])
+
+def crop_border_by_color(img, fillval=None, thresh=0, channel=None):
+    r"""
+    Crops an image to remove any constant color padding around the edges.
+
+    Args:
+        img (np.ndarray):
+            Input image array of shape (H, W) or (H, W, C).
+
+        fillval (np.ndarray | None, optional):
+            The color to be treated as padding and removed.
+            If None, defaults to white (255 for each channel).
+
+        thresh (int, optional):
+            Allowable deviation from `fillval` when determining padding.
+            Higher values allow near-matching colors to be considered padding.
+            Defaults to 0.
+
+        channel (int | None, optional):
+            If specified, only considers the given channel for distance calculation.
+            Defaults to None (all channels are considered).
+
+    Returns:
+        ndarray: cropped_img
+
+    Example:
+        >>> from kwimage.im_core import *  # NOQA
+        >>> img = np.full((10, 10, 3), 255, dtype=np.uint8)
+        >>> img[2:8, 2:8] = 0  # Add black content in the middle
+        >>> cropped = crop_border_by_color(img)
+        >>> assert cropped.shape == (6, 6, 3)
+    """
+    import kwimage
+    import numpy as np
+    if fillval is None:
+        fillval = np.array([255] * kwimage.num_channels(img))
+    # for colored images
+    pixel = fillval
+    dist = _get_pixel_dist(img, pixel, channel=channel)
+    isfill = dist <= thresh
+    # isfill should just be 2D
+    # Fix shape that comes back as (1, W, H)
+    if len(isfill.shape) == 3 and isfill.shape[0] == 1:
+        if np.all(np.greater(isfill.shape[1:2], [4, 4])):
+            isfill = isfill[0]
+    rowslice, colslice = _get_crop_slices(isfill)
+    cropped_img = img[rowslice, colslice]
+    return cropped_img
+
+
+def _get_pixel_dist(img, pixel, channel=None):
+    """
+    Computes the absolute difference between an image and a reference pixel
+    value.
+
+    Args:
+        img (np.ndarray):
+            Input image array of shape (H, W) or (H, W, C).
+
+        pixel (np.ndarray):
+            Reference pixel value to compare against.
+            Should have shape (C,) or (1, 1, C) for compatibility.
+
+        channel (int | None, optional):
+            If specified, computes the difference only for the given channel.
+            Defaults to None (computes sum across all channels).
+
+    Returns:
+        np.ndarray:
+            An array of shape (H, W) representing the per-pixel distance.
+
+    Example:
+        >>> from kwimage.im_core import _get_pixel_dist
+        >>> import numpy as np
+        >>> img = np.random.rand(256, 256, 3)
+        >>> pixel = np.random.rand(3)
+        >>> channel = None
+        >>> _get_pixel_dist(img, pixel, channel)
+    """
+    import kwimage
+    import numpy as np
+    pixel = np.asarray(pixel)
+    if len(pixel.shape) < 2:
+        pixel = pixel[None, None, :]
+    img, pixel = kwimage.make_channels_comparable(img, pixel)
+    dist = np.abs(img - pixel)
+    if len(img.shape) > 2:
+        if channel is None:
+            dist = np.sum(dist, axis=2)
         else:
-            reorg_mask = mask.swapaxes(0, axis)
-            parts = []
-            for item, item_mask in zip(reorg, reorg_mask):
-                part = normalize_intensity(item, nodata=nodata, axis=None,
-                                           mask=item_mask)
-                parts.append(part[None, :])
-        recomb = np.concatenate(parts, axis=0)
-        final = recomb.swapaxes(0, axis)
-        return final
+            dist = dist[:, :, channel]
+    return dist
 
-    if imdata.dtype.kind == 'f':
-        if mask is None:
-            mask = ~np.isnan(imdata)
 
-    if mask is None:
-        if nodata is not None:
-            mask = imdata != nodata
+def _get_crop_slices(isfill):
+    r"""
+    Determines the crop slices needed to remove the filled border.
 
-    if mask is None:
-        imdata_valid = imdata
-    else:
-        imdata_valid = imdata[mask]
+    Args:
+        isfill (np.ndarray):
+            A boolean mask of shape (H, W) indicating fill regions.
 
-    assert not np.any(np.isnan(imdata_valid))
+    Returns:
+        tuple[slice, slice]:
+            Slices to apply on rows and columns to crop the image.
 
-    normalizer = find_robust_normalizers(imdata_valid, params=params)
+    Example:
+        >>> from kwimage.im_core import _get_crop_slices
+        >>> isfill = np.array([
+        ...     [1, 1, 1, 1, 1],
+        ...     [1, 0, 0, 0, 1],
+        ...     [1, 0, 0, 0, 1],
+        ...     [1, 1, 1, 1, 1],
+        ... ])
+        >>> rowslice, colslice = _get_crop_slices(isfill)
+        >>> assert rowslice.start == 1 and rowslice.stop == 3
+        >>> assert colslice.start == 1 and colslice.stop == 4
+        >>> ###
+        >>> isfill = np.array([
+        ...     [1, 1, 0, 1, 1],
+        ...     [0, 0, 0, 0, 1],
+        ...     [1, 0, 0, 0, 0],
+        ...     [1, 1, 1, 0, 1],
+        ... ])
+        >>> rowslice, colslice = _get_crop_slices(isfill)
+        >>> assert rowslice.start == 0 and rowslice.stop == 4
+        >>> assert colslice.start == 0 and colslice.stop == 5
+        >>> ###
+        >>> from kwimage.im_core import *  # NOQA
+        >>> isfill = np.array([
+        ...     [1, 1, 1, 1, 1],
+        ...     [1, 1, 1, 1, 1],
+        ...     [1, 1, 0, 1, 1],
+        ...     [1, 1, 0, 1, 1],
+        ...     [1, 1, 1, 1, 1],
+        ...     [1, 1, 1, 1, 1],
+        ... ])
+        >>> rowslice, colslice = _get_crop_slices(isfill)
+        >>> assert rowslice.start == 2 and rowslice.stop == 4
+        >>> assert colslice.start == 2 and colslice.stop == 3
+    """
+    import numpy as np
+    import kwarray
+    from functools import reduce
+    fill_colxs = [np.where(row)[0] for row in isfill]
+    fill_rowxs = [np.where(col)[0] for col in isfill.T]
+    nRows, nCols = isfill.shape[0:2]
+    filled_columns = reduce(np.intersect1d, fill_colxs)
+    filled_rows = reduce(np.intersect1d, fill_rowxs)
 
-    if normalizer['type'] is None:
-        imdata_normalized = imdata.astype(dtype)
-    elif normalizer['type'] == 'normalize':
-        # Note: we are using kwarray normalize, the one in kwimage is deprecated
-        imdata_valid_normalized = kwarray.normalize(
-            imdata_valid.astype(dtype), mode=normalizer['mode'],
-            beta=normalizer['beta'], alpha=normalizer['alpha'],
-        )
-        if mask is None:
-            imdata_normalized = imdata_valid_normalized
-        else:
-            imdata_normalized = imdata.copy()
-            imdata_normalized[mask] = imdata_valid_normalized
-    else:
-        raise KeyError(normalizer['type'])
+    consec_rows_list = kwarray.group_consecutive(filled_rows)
+    consec_cols_list = kwarray.group_consecutive(filled_columns)
 
-    if mask is not None:
-        result = np.where(mask, imdata_normalized, imdata)
-    else:
-        result = imdata_normalized
+    def get_consec_endpoint(consec_index_list, endpoint):
+        """
+        consec_index_list = consec_cols_list
+        endpoint = 0
+        """
+        for consec_index in consec_index_list:
+            if np.any(np.array(consec_index) == endpoint):
+                return consec_index
 
-    if return_info:
-        return result, normalizer
-    else:
-        return result
+    def get_min_consec_endpoint(consec_rows_list, endpoint):
+        consec_index = get_consec_endpoint(consec_rows_list, endpoint)
+        if consec_index is None:
+            return endpoint
+        return max(consec_index) + 1
+
+    def get_max_consec_endpoint(consec_rows_list, endpoint):
+        consec_index = get_consec_endpoint(consec_rows_list, endpoint)
+        if consec_index is None:
+            return endpoint + 1
+        return min(consec_index)
+
+    consec_rows_top    = get_min_consec_endpoint(consec_rows_list, 0)
+    consec_rows_bottom = get_max_consec_endpoint(consec_rows_list, nRows - 1)
+    remove_cols_left   = get_min_consec_endpoint(consec_cols_list, 0)
+    remove_cols_right  = get_max_consec_endpoint(consec_cols_list, nCols - 1)
+    rowslice = slice(consec_rows_top, consec_rows_bottom)
+    colslice = slice(remove_cols_left, remove_cols_right)
+    return rowslice, colslice
