@@ -22,6 +22,7 @@ If you want to visualize boxes and scores you can do this:
     >>>     class_idxs=class_idxs,
     >>>     classes=classes,
     >>> )
+    >>> # xdoctest: +REQUIRES(module:kwplot)
     >>> dets.draw()
     >>> import matplotlib.pyplot as plt
     >>> plt.gca().set_xlim(0, 224)
@@ -30,27 +31,14 @@ If you want to visualize boxes and scores you can do this:
 """
 import numpy as np
 import ubelt as ub
+import sys
 from kwimage.structs import boxes as _boxes
 from kwimage.structs import _generic
 
-try:
-    from packaging.version import parse as LooseVersion
-except ImportError:
-    from distutils.version import LooseVersion
-
-
-try:
-    from xdev import profile
-except Exception:
-    from ubelt import identity as profile
-
-try:
-    import torch
-except Exception:
-    torch = None
-    _TORCH_HAS_BOOL_COMP = False
-else:
-    _TORCH_HAS_BOOL_COMP = LooseVersion(torch.__version__) >= LooseVersion('1.2.0')
+# try:
+#     import torch
+# except Exception:
+#     torch = None
 
 
 class _DetDrawMixin:
@@ -64,18 +52,18 @@ class _DetDrawMixin:
         Draws boxes using matplotlib
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
             >>> self = Detections.random(num=10, scale=512.0, rng=0, classes=['a', 'b', 'c'])
             >>> self.boxes.translate((-128, -128), inplace=True)
             >>> image = (np.random.rand(256, 256) * 255).astype(np.uint8)
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> import kwplot
             >>> kwplot.autompl()
             >>> fig = kwplot.figure(fnum=1, doclf=True)
             >>> kwplot.imshow(image)
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> self.draw(color='blue', alpha=None)
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> for o in fig.findobj():  # http://matplotlib.1069221.n5.nabble.com/How-to-turn-off-all-clipping-td1813.html
             >>>     o.set_clip_on(False)
             >>> kwplot.show_if_requested()
@@ -153,46 +141,80 @@ class _DetDrawMixin:
             xdoctest -m kwimage.structs.detections _DetDrawMixin.draw_on:1 --profile --show
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
+            >>> import kwimage
             >>> import kwplot
-            >>> self = Detections.random(num=10, scale=512, rng=0)
+            >>> self = kwimage.Detections.random(num=10, scale=512, rng=0)
             >>> image = (np.random.rand(512, 512) * 255).astype(np.uint8)
             >>> image2 = self.draw_on(image, color='blue')
-            >>> # xdoc: +REQUIRES(--show)
-            >>> kwplot.figure(fnum=2000, doclf=True)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> kwplot.autompl()
-            >>> kwplot.imshow(image2)
+            >>> kwplot.figure(fnum=2000, doclf=True)
+            >>> kwplot.imshow(image2, title='test basic detection drawing')
             >>> kwplot.show_if_requested()
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
+            >>> import kwimage
             >>> from kwimage.structs.detections import *  # NOQA
             >>> import kwplot
-            >>> self = Detections.random(num=10, scale=512, rng=0)
+            >>> self = kwimage.Detections.random(num=10, scale=512, rng=0)
             >>> image = (np.random.rand(512, 512) * 255).astype(np.uint8)
             >>> image2 = self.draw_on(image, color='classes')
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> kwplot.autompl()
+            >>> kwplot.figure(fnum=2000, doclf=True)
+            >>> kwplot.imshow(image2, title='test color by classes')
+            >>> kwplot.show_if_requested()
+
+        Example:
+            >>> # xdoctest: +REQUIRES(module:kwplot)
+            >>> # xdoctest: +REQUIRES(--profile)
+            >>> import kwimage
+            >>> import kwplot
+            >>> self = kwimage.Detections.random(num=100, scale=512, rng=0, keypoints=True, segmentations=True)
+            >>> image = (np.random.rand(512, 512) * 255).astype(np.uint8)
+            >>> image2 = self.draw_on(image, color='blue')
+            >>> # xdoctest: +REQUIRES(--show)
             >>> kwplot.figure(fnum=2000, doclf=True)
             >>> kwplot.autompl()
+            >>> kwplot.imshow(image2, title='test keypoints and segmentations')
+            >>> kwplot.show_if_requested()
+
+        Example:
+            >>> # xdoctest: +REQUIRES(module:kwplot)
+            >>> # Test that boxes with tiny scores are drawn correctly
+            >>> import kwimage
+            >>> import kwplot
+            >>> self = kwimage.Detections.random(num=10, scale=512, rng=0)
+            >>> self.data['scores'][:] = 1.23e-8
+            >>> image = (np.random.rand(512, 512) * 255).astype(np.uint8)
+            >>> image2 = self.draw_on(image, color='blue')
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> kwplot.autompl()
+            >>> kwplot.figure(fnum=2000, doclf=True, title='test tiny scores')
             >>> kwplot.imshow(image2)
             >>> kwplot.show_if_requested()
 
         Example:
-            >>> # xdoc: +REQUIRES(module:kwplot)
-            >>> # xdoc: +REQUIRES(--profile)
+            >>> # xdoctest: +REQUIRES(module:kwplot)
+            >>> # Test that alpha can be passed as an iterable.
+            >>> import kwimage
             >>> import kwplot
-            >>> self = Detections.random(num=100, scale=512, rng=0, keypoints=True, segmentations=True)
-            >>> image = (np.random.rand(512, 512) * 255).astype(np.uint8)
-            >>> image2 = self.draw_on(image, color='blue')
-            >>> # xdoc: +REQUIRES(--show)
-            >>> kwplot.figure(fnum=2000, doclf=True)
+            >>> self = kwimage.Detections.random(num=3, scale=512, rng=0)
+            >>> canvas = self.draw_on(None, color='blue', alpha=[0, 0.5, 1.0])
+            >>> unique_values = np.unique(canvas)
+            >>> assert len(unique_values) == 3, ub.paragraph(
+                '''2 non-zero alpha colors plus background should have 3 unique
+                colors''')
+            >>> # xdoctest: +REQUIRES(--show)
             >>> kwplot.autompl()
-            >>> kwplot.imshow(image2)
+            >>> kwplot.imshow(canvas, fnum=2000, doclf=True, title='test custom alphas')
             >>> kwplot.show_if_requested()
 
         Ignore:
-            # xdoc: +REQUIRES(module:kwplot)
-            # xdoc: +REQUIRES(--profile)
+            # xdoctest: +REQUIRES(module:kwplot)
+            # xdoctest: +REQUIRES(--profile)
             import kwplot
 
             import kwimage
@@ -201,7 +223,7 @@ class _DetDrawMixin:
             self = self.scale(2e6)
             image2 = self.draw_on(image, color='blue')
 
-            >>> # xdoc: +REQUIRES(--show)
+            >>> # xdoctest: +REQUIRES(--show)
             >>> kwplot.figure(fnum=2000, doclf=True)
             >>> kwplot.autompl()
             >>> kwplot.imshow(image2)
@@ -226,25 +248,27 @@ class _DetDrawMixin:
 
         dtype_fixer = _generic._consistent_dtype_fixer(image)
 
-        segmentations = self.data.get('segmentations', None)
-        if sseg and segmentations is not None:
-            if ssegkw is None:
-                ssegkw = {
-                    'alpha': 0.4,
-                    'color': color,
-                }
-            image = segmentations.draw_on(image, **ssegkw)
+        if len(self):
 
-        if boxes:
-            image = self.boxes.draw_on(image, color=color, alpha=alpha,
-                                       labels=labels, label_loc=label_loc,
-                                       thickness=thickness)
+            segmentations = self.data.get('segmentations', None)
+            if sseg and segmentations is not None:
+                if ssegkw is None:
+                    ssegkw = {
+                        'alpha': 0.4,
+                        'color': color,
+                    }
+                image = segmentations.draw_on(image, **ssegkw)
 
-        keypoints = self.data.get('keypoints', None)
-        if kpts and keypoints is not None:
-            # image = kwimage.ensure_float01(image)
-            image = keypoints.draw_on(image, radius=radius, color=color)
-            # kwimage.ensure_float01(image)
+            if boxes:
+                image = self.boxes.draw_on(image, color=color, alpha=alpha,
+                                           labels=labels, label_loc=label_loc,
+                                           thickness=thickness)
+
+            keypoints = self.data.get('keypoints', None)
+            if kpts and keypoints is not None:
+                # image = kwimage.ensure_float01(image)
+                image = keypoints.draw_on(image, radius=radius, color=color)
+                # kwimage.ensure_float01(image)
 
         image = dtype_fixer(image, copy=False)
         return image
@@ -299,8 +323,10 @@ class _DetDrawMixin:
         Either passes through user specified alpha or chooses a sensible
         default
         """
-        if alpha in ['score', 'scores']:
+        if isinstance(alpha, str) and alpha in ['score', 'scores']:
             alpha = np.sqrt(self.scores)
+        elif ub.iterable(alpha):
+            assert len(alpha) == self.num_boxes()
         else:
             if alpha is None or alpha is False:
                 alpha = 1.0
@@ -316,10 +342,12 @@ class _DetDrawMixin:
             return float('nan') if s is None else s
 
         if labels:
-            if labels is True:
+            if labels is True or isinstance(labels, int) and labels:
                 parts = []
                 if self.data.get('class_idxs', None) is not None:
                     parts.append('class')
+                elif self.data.get('cids', None) is not None:
+                    parts.append('class')
                 # Choose sensible default
                 if self.data.get('scores', None) is not None:
                     parts.append('score')
@@ -327,10 +355,20 @@ class _DetDrawMixin:
 
             if isinstance(labels, str):
                 if labels in ['class', 'class+score']:
-                    if self.classes:
-                        identifers = list(ub.take(self.classes, self.class_idxs))
+                    if 'class_idxs' in self.data:
+                        if self.classes:
+                            identifers = ['None' if cx is None else self.classes[cx] for cx in self.class_idxs]
+                        else:
+                            identifers = ['cx={}'.format(cx) for cx in self.class_idxs]
+                    elif 'cids' in self.data:
+                        if self.classes and hasattr(self.classes, 'id_to_node'):
+                            identifers = ['None' if cid is None else self.classes.id_to_node[cid] for cid in self.data['cids']]
+                        else:
+                            identifers = ['cid={}'.format(cid) for cid in self.data['cids']]
                     else:
-                        identifers = self.class_idxs
+                        # Cant determine label for class
+                        identifers = ['?' for _ in range(len(self))]
+
                 if labels in ['class']:
                     labels = identifers
                 elif labels in ['score']:
@@ -359,16 +397,41 @@ class _DetAlgoMixin:
                 threshold. Higher values are are more permissive (more boxes
                 are returned). A value of 0 means that returned boxes will have
                 no overlap.
+
             perclass (bool): if True, works on a per-class basis
-            impl (str): nms implementation to use
+
+            impl (str): implementation can be "auto", "python", "cython_cpu",
+                "gpu", "torch", or "torchvision". Not all backends may be
+                available, see :func:`kwimage.algo.algo_nms.available_nms_impls`
+                for what is supported on your system.
+
             daq (bool | Dict): if False, uses reqgular nms, otherwise uses
-                divide and conquor algorithm. If `daq` is a Dict, then
+                divide and conquer algorithm. If `daq` is a Dict, then
                 it is used as the kwargs to `kwimage.daq_spatial_nms`
 
             device_id : try not to use. only used if impl is gpu
 
         Returns:
             ndarray[Shape['*'], Integer]: indices of boxes to keep
+
+        SeeAlso:
+            :func:`kwimage.algo.algo_nms.non_max_supression`.
+
+        Example:
+            >>> import kwimage
+            >>> dets1 = kwimage.Detections.random(rng=0).scale((512, 512))
+            >>> keep = dets1.non_max_supression(thresh=0.2)
+            >>> dets2 = dets1.take(keep)
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> import kwplot
+            >>> kwplot.autompl()
+            >>> canvas = np.zeros((512, 512, 3))
+            >>> canvas1 = dets1.draw_on(canvas.copy())
+            >>> canvas2 = dets2.draw_on(canvas.copy())
+            >>> kwplot.figure(fnum=1, pnum=(1, 2, 1))
+            >>> kwplot.imshow(canvas1)
+            >>> kwplot.figure(fnum=1, pnum=(1, 2, 2))
+            >>> kwplot.imshow(canvas2)
         """
         import kwimage
         classes = self.class_idxs if perclass else None
@@ -463,7 +526,7 @@ class _DetAlgoMixin:
         if img_dims is None:
             img_dims = np.array(input_dims)
         # print(fcn_target.keys())
-        # print('fcn_target: ' + ub.repr2(ub.map_vals(lambda x: x.shape, fcn_target), nl=1))
+        # print('fcn_target: ' + ub.urepr(ub.map_vals(lambda x: x.shape, fcn_target), nl=1))
 
         impl = kwarray.ArrayAPI.coerce(fcn_target['cidx'])
 
@@ -482,7 +545,8 @@ class _DetAlgoMixin:
                 sigma = 0.3 * ((k - 1) * 0.5 - 1) + 0.8  # opencv formula
                 data = impl.contiguous(class_probs.T)
                 import cv2
-                cv2.GaussianBlur(data, (k, k), sigma, dst=data)
+                if data.size > 0:
+                    cv2.GaussianBlur(data, (k, k), sigma, dst=data)
                 class_probs = impl.contiguous(data.T)
 
             if soften > 1:
@@ -518,7 +582,7 @@ class _DetAlgoMixin:
                 kw_heat['kpts_ignore'] = fcn_target['kpts_ignore']
 
         self = kwimage.Heatmap(**kw_heat)
-        # print('self.data: ' + ub.repr2(ub.map_vals(lambda x: x.shape, self.data), nl=1))
+        # print('self.data: ' + ub.urepr(ub.map_vals(lambda x: x.shape, self.data), nl=1))
         return self
 
 
@@ -648,6 +712,7 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
             >>> assert other.data is self.data, 'try not to copy unless necessary'
 
         """
+        torch = sys.modules.get('torch', None)
         # Standardize input format
         if kwargs:
             if data or meta:
@@ -677,8 +742,10 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
                 ### Make it easier to specify keypoints and segmentations
                 if 'segmentations' in data:
                     import kwimage
-                    data['segmentations'] = kwimage.SegmentationList.coerce(
-                        data['segmentations'])
+                    segmentations = data.get('segmentations', None)
+                    if segmentations is not None:
+                        segmentations = kwimage.SegmentationList.coerce(segmentations)
+                        data['segmentations'] = segmentations
 
                 for k, v in data.items():
                     if v is None:
@@ -849,6 +916,38 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
             >>> dets = kwimage.Detections.from_coco_annots(
             >>>     anns, sampler.dset.dataset['categories'], sampler.catgraph,
             >>>     kp_classes, shape=shape)
+
+        Example:
+            >>> from kwimage.structs.detections import *  # NOQA
+            >>> # xdoctest: +REQUIRES(--module:ndsampler)
+            >>> kpclasses = kwcoco.CategoryTree.from_coco([
+            >>>     {'name': 'head', 'id': 1},
+            >>>     {'name': 'backbone', 'id': 2},
+            >>>     {'name': 'stem', 'id': 3},
+            >>> ])
+            >>> points = kwimage.Points.random(3, classes=kpclasses)
+            >>> anns = [{
+            >>>     'id': 0,
+            >>>     'image_id': 1,
+            >>>     'category_id': 2,
+            >>>     'bbox': [2, 3, 10, 10],
+            >>>     'keypoints': points.to_coco('new-v2'),
+            >>> }]
+            >>> dataset = {
+            >>>     'images': [],
+            >>>     'annotations': anns,
+            >>>     'categories': [
+            >>>         {'id': 0, 'name': 'background'},
+            >>>         {'id': 2, 'name': 'class1', 'keypoint_skeleton': [('head', 'backbone'), ('backbone', 'stem')]}
+            >>>     ],
+            >>>     'keypoint_categories': list(kpclasses.to_coco()),
+            >>> }
+            >>> #import ndsampler
+            >>> #dset = ndsampler.CocoDataset(dataset)
+            >>> cats = dataset['categories']
+            >>> dets = Detections.from_coco_annots(anns, cats, kp_classes=kpclasses)
+            >>> # TODO: draw a skeleton
+            >>> dets.data['keypoints'].data[0].meta
         """
         import kwimage
         cnames = None
@@ -857,12 +956,13 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
                 classes = dset.object_categories()
             except Exception:
                 pass
-            cats = dset.dataset['categories']
+            cats = dset.dataset['categories']  # redundant with classes, can simplify
             try:
                 kp_classes = dset.keypoint_categories()
             except Exception:
                 pass
-                # kp_classes = None
+            if len(kp_classes) == 0:
+                kp_classes = None
         else:
             if cats is None:
                 cnames = []
@@ -921,7 +1021,10 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
             ]
             dets.data['segmentations'] = kwimage.PolygonList(masks)
 
-        if True:
+        HANDLE_KEYPOINTS = 1
+        if HANDLE_KEYPOINTS:
+            # TODO: loop a skeleton for the Points object.
+
             name_to_cat = {c['name']: c for c in cats}
             def _lookup_kp_class_idxs(cid):
                 kpnames = None
@@ -943,21 +1046,34 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
                     kpts.append(k)
                 elif len(k) == 0:
                     kpcidxs = []
+                    kpts.append(None)
                 else:
                     kpcidxs = None
                     # TODO: correctly handle newstyle keypoints
                     if dset is not None:
                         pass
                     kpcidxs = None
-                    if not (isinstance(k, list) and len(k) and isinstance(ub.peek(k), dict)):
+                    is_oldstyle = (isinstance(k, list) and len(k) and not isinstance(ub.peek(k), dict))
+                    if is_oldstyle:
                         # oldstyle
                         if kp_classes is not None:
                             # These are only needed for old-style coco
-                            kpcidxs = _lookup_kp_class_idxs(ann['category_id'])
+                            cid = ann['category_id']
+                            try:
+                                kpcidxs = _lookup_kp_class_idxs(cid)
+                            except KeyError:
+                                # unable to find keypoint category ids
+                                ...
 
                     pts = kwimage.Points.from_coco(
                         k, class_idxs=kpcidxs, classes=kp_classes)
                     kpts.append(pts)
+
+            if __debug__:
+                boxes = dets.data.get('boxes', None)
+                if boxes is not None:
+                    assert len(kpts) == len(boxes)
+
             dets.data['keypoints'] = kwimage.PointsList(kpts)
 
             if kp_classes is not None:
@@ -1007,13 +1123,15 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
         """
         import kwarray
         to_collate = {}
-        if 'boxes' in self.data:
-            to_collate['bbox'] = list(self.data['boxes'].to_coco(style=style))
+        boxes = self.data.get('boxes', None)
+        if boxes is not None:
+            to_collate['bbox'] = list(boxes.to_coco(style=style))
 
-        if 'class_idxs' in self.data:
+        class_idxs = self.data.get('class_idxs', None)
+        if class_idxs is not None:
             if 'classes' in self.meta:
                 classes = self.meta['classes']
-                catnames = [classes[cidx] for cidx in self.class_idxs]
+                catnames = [classes[cidx] for cidx in class_idxs]
                 if cname_to_cat is not None:
                     pass
                 if dset is not None:
@@ -1026,29 +1144,36 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
                     raise NotImplementedError(
                         'Passed a dset to resolve category id, but this '
                         'detection object has no classes meta attribute')
-                to_collate['category_index'] = kwarray.ArrayAPI.tolist(
-                    self.data['class_idxs'])
+                to_collate['category_index'] = kwarray.ArrayAPI.tolist(class_idxs)
 
-        if 'keypoints' in self.data:
-            to_collate['keypoints'] = list(self.data['keypoints'].to_coco(
-                style=style))
+        keypoints = self.data.get('keypoints', None)
+        if keypoints is not None:
+            to_collate['keypoints'] = list(keypoints.to_coco(style=style))
 
-        if 'segmentations' in self.data:
-            to_collate['segmentation'] = list(self.data['segmentations'].to_coco(
-                style=style))
+        segmentations = self.data.get('segmentations', None)
+        if segmentations is not None:
+            to_collate['segmentation'] = list(segmentations.to_coco(style=style))
 
-        if 'scores' in self.data:
-            to_collate['score'] = kwarray.ArrayAPI.tolist(self.data['scores'])
+        scores = self.data.get('scores', None)
+        if scores is not None:
+            to_collate['score'] = kwarray.ArrayAPI.tolist(scores)
 
-        if 'weights' in self.data:
-            to_collate['weight'] = kwarray.ArrayAPI.tolist(self.data['weights'])
+        weights = self.data.get('weights', None)
+        if weights is not None:
+            to_collate['weight'] = kwarray.ArrayAPI.tolist(weights)
 
-        if 'probs' in self.data:
-            to_collate['prob'] = kwarray.ArrayAPI.tolist(self.data['probs'])
+        probs = self.data.get('probs', None)
+        if probs is not None:
+            to_collate['prob'] = kwarray.ArrayAPI.tolist(probs)
 
         if image_id is not None:
             to_collate['image_id'] = [image_id] * len(self)
 
+        if __debug__:
+            # Error before we silently return bad data
+            collate_lens = ub.udict(to_collate).map_values(len)
+            assert ub.allsame(collate_lens.values())
+
         keys = list(to_collate.keys())
         for item_vals in zip(*to_collate.values()):
             ann = ub.dzip(keys, item_vals)
@@ -1130,7 +1255,6 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
                 inplace=inplace)
         return new
 
-    @profile
     def scale(self, factor, output_dims=None, inplace=False):
         """
         Spatially scale the detections.
@@ -1153,7 +1277,6 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
                 factor, output_dims=output_dims, inplace=inplace)
         return new
 
-    @profile
     def translate(self, offset, output_dims=None, inplace=False):
         """
         Spatially translate the detections.
@@ -1231,6 +1354,7 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
         """
         sortx = self.scores.argsort()
         if reverse:
+            torch = sys.modules.get('torch', None)
             if torch is not None and torch.is_tensor(sortx):
                 sortx = torch.flip(sortx, dims=(0,))
             else:
@@ -1278,23 +1402,17 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
             raise IndexError('compress must get a flag for every item')
 
         if self.is_tensor():
+            torch = sys.modules.get('torch', None)
             if isinstance(flags, np.ndarray):
                 if flags.dtype.kind == 'b':
                     flags = flags.astype(np.uint8)
             if isinstance(flags, torch.Tensor):
-                if _TORCH_HAS_BOOL_COMP:
-                    if flags.dtype != torch.bool:
-                        flags = flags.bool()
-                else:
-                    if flags.dtype != torch.uint8:
-                        flags = flags.byte()
+                if flags.dtype != torch.bool:
+                    flags = flags.bool()
                 if flags.device != flags.device:
                     flags = flags.to(self.device)
             else:
-                if _TORCH_HAS_BOOL_COMP:
-                    flags = torch.BoolTensor(flags).to(self.device)
-                else:
-                    flags = torch.ByteTensor(flags).to(self.device)
+                flags = torch.BoolTensor(flags).to(self.device)
         newdata = {k: _generic._safe_compress(v, flags, axis)
                    for k, v in self.data.items()}
         return self.__class__(newdata, self.meta)
@@ -1319,6 +1437,7 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
             >>> assert len(subset) == 4
         """
         if self.is_tensor():
+            torch = sys.modules.get('torch', None)
             indices = torch.LongTensor(indices).to(self.device)
         newdata = {k: _generic._safe_take(v, indices, axis)
                    for k, v in self.data.items()}
@@ -1383,6 +1502,7 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
             if val is None:
                 newval = val
             else:
+                torch = sys.modules.get('torch', None)
                 if torch is not None and torch.is_tensor(val):
                     newval = val.data.cpu().numpy()
                 elif hasattr(val, 'numpy'):
@@ -1433,6 +1553,7 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
             elif hasattr(val, 'tensor'):
                 newval = val.tensor(device)
             else:
+                torch = sys.modules.get('torch', None)
                 if torch is not None and torch.is_tensor(val):
                     newval = val
                 else:
@@ -1477,7 +1598,11 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
                 Defaults to False.
             tensor (bool): determines backend.
                 DEPRECATED.  Call ``.tensor()`` on resulting object instead.
-            rng (np.random.RandomState): random state
+            rng (int | RandomState | None):
+                random state or seed
+
+        Returns:
+            Detections: random detections
 
         Example:
             >>> import kwimage
@@ -1498,8 +1623,8 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
             >>> print('dets = {}'.format(dets))
             dets = <Detections(10)>
             >>> dets.data['boxes'].quantize(inplace=True)
-            >>> print('dets.data = {}'.format(ub.repr2(
-            >>>     dets.data, nl=1, with_dtype=False, strvals=True)))
+            >>> print('dets.data = {}'.format(ub.urepr(
+            >>>     dets.data, nl=1, with_dtype=False, strvals=True, sort=1)))
             dets.data = {
                 'boxes': <Boxes(xywh,
                              array([[548, 544,  55, 172],
@@ -1511,7 +1636,7 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
                                     [118, 639,  26, 306],
                                     [264, 414, 258, 361],
                                     [ 18, 568, 439,  50],
-                                    [612, 616, 332,  66]], dtype=int32))>,
+                                    [612, 616, 332,  66]]...))>,
                 'class_idxs': [1, 2, 0, 0, 2, 0, 0, 0, 0, 0],
                 'keypoints': <PointsList(n=10)>,
                 'scores': [0.3595079 , 0.43703195, 0.6976312 , 0.06022547, 0.66676672, 0.67063787,0.21038256, 0.1289263 , 0.31542835, 0.36371077],
@@ -1585,6 +1710,11 @@ class Detections(ub.NiceRepr, _DetAlgoMixin, _DetDrawMixin):
         self = self.scale(scale)
 
         if tensor:
+            ub.schedule_deprecation(
+                'kwimage', 'tensor', 'argument to Detections.random',
+                migration='use .tensor() instead',
+                deprecate='0.8.0', error='1.0.0', remove='1.1.0'
+            )
             self = self.tensor()
 
         return self
@@ -1631,7 +1761,7 @@ def _dets_to_fcmaps(dets, bg_size, input_dims, bg_idx=0, pmin=0.6, pmax=1.0,
         >>> bg_idxs = sampler.catgraph.index('background')
         >>> fcn_target = _dets_to_fcmaps(dets, bg_size, input_dims, bg_idxs)
         >>> fcn_target.keys()
-        >>> print('fcn_target: ' + ub.repr2(ub.map_vals(lambda x: x.shape, fcn_target), nl=1))
+        >>> print('fcn_target: ' + ub.urepr(ub.map_vals(lambda x: x.shape, fcn_target), nl=1, sort=1))
         fcn_target: {
             'cidx': (512, 512),
             'class_probs': (10, 512, 512),
@@ -1681,6 +1811,7 @@ def _dets_to_fcmaps(dets, bg_size, input_dims, bg_idx=0, pmin=0.6, pmax=1.0,
         kwplot.show_if_requested()
     """
     import cv2
+    import kwimage
     # In soft mode we made a one-channel segmentation target mask
     cidx_mask = np.full(input_dims, dtype=np.int32, fill_value=bg_idx)
 
@@ -1702,7 +1833,6 @@ def _dets_to_fcmaps(dets, bg_size, input_dims, bg_idx=0, pmin=0.6, pmax=1.0,
 
     cxywh = dets.boxes.to_cxywh().data
     class_idxs = dets.class_idxs
-    import kwimage
 
     if 'segmentations' in dets.data:
         sseg_list = [None if p is None else p.to_mask(input_dims)
@@ -1771,11 +1901,13 @@ def _dets_to_fcmaps(dets, bg_size, input_dims, bg_idx=0, pmin=0.6, pmax=1.0,
 
         if sseg_mask is None:
             mask = np.zeros_like(cidx_mask, dtype=np.uint8)
-            mask = cv2.ellipse(mask, center, axes, angle=0.0,
-                               startAngle=0.0, endAngle=360.0, color=1,
-                               thickness=-1).astype(np.bool)
+            if mask.size > 0:
+                mask = cv2.ellipse(mask, center, axes, angle=0.0,
+                                   startAngle=0.0, endAngle=360.0, color=1,
+                                   thickness=-1)
+            mask = mask.astype(bool)
         else:
-            mask = sseg_mask.to_c_mask().data.astype(np.bool)
+            mask = sseg_mask.to_c_mask().data.astype(bool)
         # class index
         cidx_mask[mask] = int(cidx)
         if 'class_probs' not in exclude:
@@ -1808,19 +1940,22 @@ def _dets_to_fcmaps(dets, bg_size, input_dims, bg_idx=0, pmin=0.6, pmax=1.0,
                     if len(_xys) > 0:
                         _cidxs = pts.data['class_idxs']
                         if _cidxs is None:
+                            # import warnings
+                            # warnings.warn('Cannot rasterize keypoints with undefined categories')
                             raise ValueError(
                                 'cannot rasterize keypoints with undefined categories')
-                        for xy, kp_cidx in zip(_xys, _cidxs):
-                            if kp_cidx < 0:
-                                import warnings
-                                warnings.warn('Cannot rasterize keypoints with unknown classes')
-                            else:
-                                kp_x, kp_y = xy
-                                kp_dx = kp_x - xcoord[mask]
-                                kp_dy = kp_y - ycoord[mask]
-                                kpts_mask[0, kp_cidx][mask] = kp_dx
-                                kpts_mask[1, kp_cidx][mask] = kp_dy
-                                kpts_ignore_mask[kp_cidx][mask] = 0
+                        else:
+                            for xy, kp_cidx in zip(_xys, _cidxs):
+                                if kp_cidx < 0:
+                                    import warnings
+                                    warnings.warn('Cannot rasterize keypoints with unknown classes')
+                                else:
+                                    kp_x, kp_y = xy
+                                    kp_dx = kp_x - xcoord[mask]
+                                    kp_dy = kp_y - ycoord[mask]
+                                    kpts_mask[0, kp_cidx][mask] = kp_dx
+                                    kpts_mask[1, kp_cidx][mask] = kp_dy
+                                    kpts_ignore_mask[kp_cidx][mask] = 0
 
     fcn_target = {
         'cidx': cidx_mask,
